@@ -2,7 +2,7 @@
  * Programmer(s): Slaven Peles @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -38,7 +38,7 @@ int main(int argc, char *argv[])
   sunindextype global_length;     /* global vector length      */
   N_Vector     U, V, X, Y, Z;     /* test vectors              */
   int          print_timing;      /* turn timing on/off        */
-  MPI_Comm     comm;              /* MPI Communicator          */
+  MPI_Comm     comm, comm2;       /* MPI Communicator          */
   int          nprocs, myid;      /* Number of procs, proc id  */
 
   Vec            xvec;            /* PETSc vector              */
@@ -57,7 +57,7 @@ int main(int argc, char *argv[])
     MPI_Abort(comm, -1);
   }
 
-  local_length = atol(argv[1]);
+  local_length = (sunindextype) atol(argv[1]);
   if (local_length < 1) {
     if (myid == 0)
       printf("ERROR: local vector length must be a positive integer \n");
@@ -83,7 +83,7 @@ int main(int argc, char *argv[])
   /* Create PETSc vector */
   VecCreate(comm, &xvec);
   VecSetSizes(xvec, local_length, global_length);
-  VecSetFromOptions(xvec);    
+  VecSetFromOptions(xvec);
 
   /* Create PETSc N_Vector wrapper and test */
   X = N_VMake_Petsc(xvec);
@@ -98,6 +98,15 @@ int main(int argc, char *argv[])
   /* Check vector ID */
   fails += Test_N_VGetVectorID(X, SUNDIALS_NVEC_PETSC, myid);
 
+  /* Check vector length */
+  fails += Test_N_VGetLength(X, myid);
+
+  /* Check vector communicator -- note that the actual communicator used by the
+     PETSc vector is not necessarily identical to the one supplied to VecCreate,
+     so we compare against the one that is actually *used* in the PETSc vector. */
+  PetscObjectGetComm((PetscObject) xvec, &comm2);
+  fails += Test_N_VGetCommunicatorMPI(X, &comm2, myid);
+
   /* Test clone functions */
   fails += Test_N_VCloneEmpty(X, myid);
   fails += Test_N_VClone(X, local_length, myid);
@@ -133,13 +142,13 @@ int main(int argc, char *argv[])
   fails += Test_N_VAbs(X, Z, local_length, myid);
   fails += Test_N_VInv(X, Z, local_length, myid);
   fails += Test_N_VAddConst(X, Z, local_length, myid);
-  fails += Test_N_VDotProd(X, Y, local_length, global_length, myid);
+  fails += Test_N_VDotProd(X, Y, local_length, myid);
   fails += Test_N_VMaxNorm(X, local_length, myid);
   fails += Test_N_VWrmsNorm(X, Y, local_length, myid);
-  fails += Test_N_VWrmsNormMask(X, Y, Z, local_length, global_length, myid);
+  fails += Test_N_VWrmsNormMask(X, Y, Z, local_length, myid);
   fails += Test_N_VMin(X, local_length, myid);
-  fails += Test_N_VWL2Norm(X, Y, local_length, global_length, myid);
-  fails += Test_N_VL1Norm(X, local_length, global_length, myid);
+  fails += Test_N_VWL2Norm(X, Y, local_length, myid);
+  fails += Test_N_VL1Norm(X, local_length, myid);
   fails += Test_N_VCompare(X, Z, local_length, myid);
   fails += Test_N_VInvTest(X, Z, local_length, myid);
   fails += Test_N_VConstrMask(X, Y, Z, local_length, myid);
@@ -163,14 +172,14 @@ int main(int argc, char *argv[])
   /* fused operations */
   fails += Test_N_VLinearCombination(U, local_length, myid);
   fails += Test_N_VScaleAddMulti(U, local_length, myid);
-  fails += Test_N_VDotProdMulti(U, local_length, global_length, myid);
+  fails += Test_N_VDotProdMulti(U, local_length, myid);
 
   /* vector array operations */
   fails += Test_N_VLinearSumVectorArray(U, local_length, myid);
   fails += Test_N_VScaleVectorArray(U, local_length, myid);
   fails += Test_N_VConstVectorArray(U, local_length, myid);
   fails += Test_N_VWrmsNormVectorArray(U, local_length, myid);
-  fails += Test_N_VWrmsNormMaskVectorArray(U, local_length, global_length, myid);
+  fails += Test_N_VWrmsNormMaskVectorArray(U, local_length, myid);
   fails += Test_N_VScaleAddMultiVectorArray(U, local_length, myid);
   fails += Test_N_VLinearCombinationVectorArray(U, local_length, myid);
 
@@ -193,17 +202,37 @@ int main(int argc, char *argv[])
   /* fused operations */
   fails += Test_N_VLinearCombination(V, local_length, myid);
   fails += Test_N_VScaleAddMulti(V, local_length, myid);
-  fails += Test_N_VDotProdMulti(V, local_length, global_length, myid);
+  fails += Test_N_VDotProdMulti(V, local_length, myid);
 
   /* vector array operations */
   fails += Test_N_VLinearSumVectorArray(V, local_length, myid);
   fails += Test_N_VScaleVectorArray(V, local_length, myid);
   fails += Test_N_VConstVectorArray(V, local_length, myid);
   fails += Test_N_VWrmsNormVectorArray(V, local_length, myid);
-  fails += Test_N_VWrmsNormMaskVectorArray(V, local_length, global_length, myid);
+  fails += Test_N_VWrmsNormMaskVectorArray(V, local_length, myid);
   fails += Test_N_VScaleAddMultiVectorArray(V, local_length, myid);
   fails += Test_N_VLinearCombinationVectorArray(V, local_length, myid);
 
+  /* local reduction operations */
+  if (myid == 0) printf("\nTesting local reduction operations:\n\n");
+
+  fails += Test_N_VDotProdLocal(X, Y, local_length, myid);
+  fails += Test_N_VMaxNormLocal(X, local_length, myid);
+  fails += Test_N_VMinLocal(X, local_length, myid);
+  fails += Test_N_VL1NormLocal(X, local_length, myid);
+  fails += Test_N_VWSqrSumLocal(X, Y, local_length, myid);
+  fails += Test_N_VWSqrSumMaskLocal(X, Y, Z, local_length, myid);
+  fails += Test_N_VInvTestLocal(X, Z, local_length, myid);
+  fails += Test_N_VConstrMaskLocal(X, Y, Z, local_length, myid);
+  fails += Test_N_VMinQuotientLocal(X, Y, local_length, myid);
+
+  /* XBraid interface operations */
+  printf("\nTesting XBraid interface operations:\n\n");
+
+  fails += Test_N_VBufSize(X, local_length, myid);
+  fails += Test_N_VBufPack(X, local_length, myid);
+  fails += Test_N_VBufUnpack(X, local_length, myid);
+
   /* Free vectors */
   N_VDestroy(X);
   N_VDestroy(Y);
@@ -216,7 +245,7 @@ int main(int argc, char *argv[])
   if (fails) {
     printf("FAIL: NVector module failed %i tests, Proc %d \n\n", fails, myid);
   } else {
-    if (myid == 0) 
+    if (myid == 0)
       printf("SUCCESS: NVector module passed all tests, Proc %d \n\n",myid);
   }
 
@@ -263,15 +292,22 @@ booleantype has_data(N_Vector X)
 
 void set_element(N_Vector X, sunindextype i, realtype val)
 {
-  Vec         xv;
-  PetscScalar *a;
+  set_element_range(X, i, i, val);
+}
+
+void set_element_range(N_Vector X, sunindextype is, sunindextype ie,
+                       realtype val)
+{
+  Vec           xv;
+  PetscScalar  *a;
+  sunindextype  i;
 
   /* extract the PETSc vector from the N_Vector wrapper */
   xv = N_VGetVector_Petsc(X);
 
-  /* set i-th element of data array */
+  /* set elements [is,ie] of the data array */
   VecGetArray(xv, &a);
-  a[i] = val;
+  for(i = is; i <= ie; i++) a[i] = val;
   VecRestoreArray(xv, &a);
 }
 
@@ -303,7 +339,7 @@ double max_time(N_Vector X, double time)
   return(maxt);
 }
 
-void sync_device()
+void sync_device(N_Vector x)
 {
   /* not running on GPU, just return */
   return;
